﻿using ProjectLainer.DrawningObjects;
using ProjectLainer.Exceptions;
using System.Numerics;

namespace ProjectLainer.Generics
{
    internal class SetGeneric<T>
    where T : class
    {
        private readonly List<T?> _places;
        public int Count => _places.Count;
        private readonly int _maxCount;
        public SetGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T?>(count);
        }

        public bool Insert(T lainer, IEqualityComparer<T?>? equal = null)
        {
            if (_places.Count == _maxCount)
            {
                throw new OverflowException();
            }
            Insert(lainer, 0, equal);
            return true;
        }

        public bool Insert(T lainer, int position, IEqualityComparer<T?>? equal = null) 
        {
            // объект есть уже в коллекции - выбросить исключение
            if (!(position >= 0 && position <= Count))
            {
                return false;
            }
            if (equal != null)
            {
                foreach (T elem in _places)
                {
                    if (equal.Equals(lainer, elem))
                    {
                        throw new Exception("такой объект уже есть");
                    }
                }
            }
            _places.Insert(position, lainer);
            return true;
        }

        public bool Remove(int position)
        {
            if(position < _maxCount && position >= 0)
            {
                if(position < Count)
                {
                    _places.RemoveAt(position);
                    return true;
                }
                else
                {
                    throw new LainerNotFoundException();
                }
            }
            return false;
        }
        public T? this[int position]
        {
            get
            {
                if(position < 0 || position >= Count)
                {
                    return null;
                }
                return _places[position];
            }
            set
            {
                if (position < 0 || position >= Count || Count >= _maxCount)
                {
                    return;
                }
                _places.Insert(position, value);
            }
        }
        public IEnumerable<T?> GetLainers(int? maxLainers = null)
        {
            for (int i = 0; i < _places.Count; ++i)
            {
                yield return _places[i];
                if (maxLainers.HasValue && i == maxLainers.Value)
                {
                    yield break;
                }
            }
        }
        public void SortSet(IComparer<T?> comparer) => _places.Sort(comparer);
    }
}

